﻿using System;
using System.Diagnostics;
using System.Text.RegularExpressions;
using System.Threading;
using Windows.Devices.Bluetooth;
using Windows.Devices.Enumeration;

namespace hcitool
{
    static class Program
    {
        static readonly ManualResetEventSlim WaitHandle = new ManualResetEventSlim(false);
        static string hex = "[0-9a-fA-F]";
        static readonly string ByteInHex = hex + hex;
        static readonly string Mac = string.Join(@"\:", ByteInHex, ByteInHex, ByteInHex, ByteInHex, ByteInHex, ByteInHex);
        static readonly Regex BtIdRegex = new Regex(@"^BluetoothLE\#BluetoothLE" + Mac + "-(?<Mac>" + Mac + ")$", RegexOptions.Compiled);

        static int Main(string[] args)
        {
            if (args.Length != 1 || !args[0].Equals("lescan", StringComparison.OrdinalIgnoreCase))
            {
                Console.WriteLine("Only supports lescan option");
                Console.WriteLine("Press Enter to exit");
                Console.ReadLine();
                return 1;
            }
            var deviceWatcher = DeviceInformation.CreateWatcher(BluetoothLEDevice.GetDeviceSelector());
            deviceWatcher.Added += DeviceWatcher_Added;
            deviceWatcher.EnumerationCompleted += DeviceWatcher_EnumerationCompleted;
            Trace.WriteLine("Enumerating devices");
            deviceWatcher.Start();
            WaitHandle.Wait();
            Trace.WriteLine("Done");
            deviceWatcher.Stop();
            Console.WriteLine("Press Enter to exit");
            Console.ReadLine();
            return 0;
        }

        private static void DeviceWatcher_EnumerationCompleted(DeviceWatcher sender, object args)
        {
            WaitHandle.Set();
        }

        private static void DeviceWatcher_Added(DeviceWatcher sender, DeviceInformation args)
        {
            var match = BtIdRegex.Match(args.Id);
            if (match.Success)
            {
                var mac = match.Groups["Mac"].Value;
                Console.WriteLine(mac + " " + args.Name);
            }
        }
    }
}
